import json
import urllib

import requests

args = {
        'client_id': 'horusclientid',#your client id here
        'client_secret': 'BljqOjqstrr4Klv9VyRwAL3Fg5bOquN8rmICYZ0qsd8=',#your azure secret here
        'scope': 'http://api.microsofttranslator.com',
        'grant_type': 'client_credentials'
    }
oauth_url = 'https://datamarket.accesscontrol.windows.net/v2/OAuth2-13'
oauth_junk = json.loads(requests.post(oauth_url, data=urllib.urlencode(args)).content)
translation_args = {
        'text': "agora nao da mais problema?",
        'to': 'en'
        }
headers={'Authorization': 'Bearer '+oauth_junk['access_token']}
translation_url = 'http://api.microsofttranslator.com/V2/Ajax.svc/Translate?'
translation_result = requests.get(translation_url+urllib.urlencode(translation_args), headers=headers)
print translation_result.content




from horus.components import Core

horus = Core(False, 5)
print horus.version_label
print horus.annotate("petropolis is more relaxing than bangu", "", 0, "../output/out", "csv")
print horus.get_cv_annotation()



import csv
import itertools
import json
import logging

import matplotlib.pyplot as plt
import numpy as np
import pandas as pd
from sklearn.metrics import confusion_matrix

from horus.components import Core


def confusion_matrix_analysis(horus_matrix):
    df = pd.DataFrame(horus_matrix)
    res = df.loc[df[4].isin(ner_ritter)]

    y = res[4]
    y_true = ['PER' if x in ner_ritter_per else 'LOC' if x in ner_ritter_loc else 'ORG' for x in y]
    y_pred_cv = res[13]
    y_pred_tx = res[20]
    cm1 = confusion_matrix(y_true, y_pred_cv)
    cm2 = confusion_matrix(y_true, y_pred_tx)

    plt.figure()
    plot_confusion_matrix(cm1, classes=['PER', 'LOC', 'ORG'], normalize=True,
                          title='CV - Normalized Confusion Matrix')
    plt.show()

    plot_confusion_matrix(cm2, classes=['PER', 'LOC', 'ORG'], normalize=True,
                          title='TX - Normalized Confusion Matrix')
    plt.show()


def plot_confusion_matrix(cm, classes,
                          normalize=False,
                          title='Confusion matrix',
                          cmap=plt.cm.Blues):
    """
    This function prints and plots the confusion matrix.
    Normalization can be applied by setting `normalize=True`.
    """
    plt.imshow(cm, interpolation='nearest', cmap=cmap)
    plt.title(title)
    plt.colorbar()
    tick_marks = np.arange(len(classes))
    plt.xticks(tick_marks, classes, rotation=45)
    plt.yticks(tick_marks, classes)

    if normalize:
        cm = cm.astype('float') / cm.sum(axis=1)[:, np.newaxis]
        print("Normalized confusion matrix")
    else:
        print('Confusion matrix, without normalization')

    print(cm)

    thresh = cm.max() / 2.
    for i, j in itertools.product(range(cm.shape[0]), range(cm.shape[1])):
        plt.text(j, i, cm[i, j],
                 horizontalalignment="center",
                 color="white" if cm[i, j] > thresh else "black")

    plt.tight_layout()
    plt.ylabel('True label')
    plt.xlabel('Predicted label')


def example_analysis(horus_matrix):
    tot_loc, tot_org, tot_per, tot_others = 0, 0, 0, 0
    hit_loc, hit_org, hit_per = 0, 0, 0
    hit_loc_tx, hit_org_tx, hit_per_tx = 0, 0, 0
    hit_loc_final, hit_org_final, hit_per_final = 0, 0, 0
    for item in horus_matrix:
        if item[0] == 1:
            #  this indicator -12 plays an important rule
            #if item[13] == 'LOC' and item[12] < 0:
            #    # get the 2nd best candidate
            #    if item[9] >= item[10]:
            #        item[13] = 'ORG'
            #    else:
            #        item[13] = 'PER'
            if item[4] in ner_ritter_per:
                tot_per += 1
                if item[13] == 'PER':
                    hit_per += 1
                if item[20] == 'PER':
                    hit_per_tx += 1
                if item[21] == 'PER':
                    hit_per_final += 1
            elif item[4] in ner_ritter_loc:
                tot_loc += 1
                if item[13] == 'LOC':
                    hit_loc += 1
                if item[20] == 'LOC':
                    hit_loc_tx += 1
                if item[21] == 'LOC':
                    hit_loc_final += 1
            elif item[4] in ner_ritter_org:
                tot_org += 1
                if item[13] == 'ORG':
                    hit_org += 1
                if item[20] == 'ORG':
                    hit_org_tx += 1
                if item[21] == 'ORG':
                    hit_org_final += 1
        else:
            tot_others += 1

    # F1 = 2 * (precision * recall) / (precision + recall)

    logging.info(':: LOC -> hits-cv: %s | %2f | hits-tx: %s | %2f | hits-final: %s | %2f || %s' % (str(hit_loc),
                                                                            float(hit_loc) / tot_loc,
                                                                            str(hit_loc_tx),
                                                                            float(hit_loc_tx) / tot_loc,
                                                                            str(hit_loc_final),
                                                                            float(hit_loc_final) / tot_loc,
                                                                            str(tot_loc)))
    logging.info(':: ORG -> hits-cv: %s | %2f | hits-tx: %s | %2f | hits-final: %s | %2f || %s' % (str(hit_org),
                                                                            float(hit_org) / tot_org,
                                                                            str(hit_org_tx),
                                                                            float(hit_org_tx) / tot_org,
                                                                            str(hit_org_final),
                                                                            float(hit_org_final) / tot_org,
                                                                            str(tot_org)))
    logging.info(':: PER -> hits-cv: %s | %2f | hits-tx: %s | %2f | hits-final: %s | %2f || %s' % (str(hit_per),
                                                                            float(hit_per) / tot_per,
                                                                            str(hit_per_tx),
                                                                            float(hit_per_tx) / tot_per,
                                                                            str(hit_per_final),
                                                                            float(hit_per_final) / tot_per,
                                                                            str(tot_per)))

    logging.info('------------------------------------------------------------------')
    logging.info(':: statistics')
    logging.info(':: number of tokens: ' + str(len(horus_matrix)))


horus = Core(5)
ner_ritter_per = ['B-person', 'I-person']
ner_ritter_org = ['B-company', 'I-company']
ner_ritter_loc = ['B-geo-loc', 'I-geo-loc']
ner_ritter = []
ner_ritter.extend(ner_ritter_per)
ner_ritter.extend(ner_ritter_org)
ner_ritter.extend(ner_ritter_loc)

with open(horus.horus_final_data_json) as json_data:
    horus_matrix = json.load(json_data)

list1 = [0 ,2]
list2=[["abc", 1, "def"], ["ghi", 2, "wxy"]]
newList = [[each_list[i] for i in list1] for each_list in list2]
# word_term, Y, klass_cv, klass_txt, final_klass
horus_light = [[each_list[i] for i in [3,4,13,20,21]] for each_list in horus_matrix]

hmetalight = open('/Users/dnes/Dropbox/Doutorado_Alemanha/#Papers/#DeFacto Files/components/cache/horus_out_light.csv', 'wb')
wr = csv.writer(hmetalight, quoting=csv.QUOTE_ALL)
wr.writerows(horus_light)

#with open('/Users/dnes/Dropbox/Doutorado_Alemanha/#Papers/#DeFacto Files/components/cache/horus_out_light.json', 'wb') as outfile1:
#    json.dump(horus_light, outfile1)

#with open(components.horus_final_data_csv, 'rb') as f:
#    reader = csv.reader(f)
#    # next(reader) # hack to ignore the header
#    horus_matrix = list(reader)


example_analysis(horus_matrix)



# -*- coding: utf-8 -*-

"""
==========================================================
HORUS: Named Entity Recognition Algorithm
==========================================================

HORUS is a Named Entity Recognition Algorithm specifically
designed for short-text, i.e., microblogs and other noisy
datasets existing on the web, e.g.: social media, some web-
sites, blogs and etc..

It is a simplistic approach based on multi-level machine
learning combined with computer vision techniques.

more info at: https://github.com/dnes85/components-models

"""

# Author: Esteves <diegoesteves@gmail.com>
# Version: 1.0
# Version Label: HORUS_NER_2016_1.0
# License: BSD 3 clause
from optparse import OptionParser

from src.horus.components.core import Core


def main():

    op = OptionParser(usage='usage: %prog [options] arguments')

    op.add_option("--input_text", dest="input_text",
                  help="The text to be annotated")

    op.add_option("--input_file", dest="input_file",
                  help="The file to be annotated")

    op.add_option("--ds_format", dest="ds_format", default=0,
                  help="The format to be annotated [0 = free text (default), 1 = Ritter]")

    op.add_option("--output_file", dest="output_file", default="horus_out",
                  help="The output file")

    op.add_option("--output_format", dest="output_format", default="json",
                  help="The output file type")

    (opts, args) = op.parse_args()
    print(__doc__)
    op.print_help()

    if not opts.input_text and not opts.input_file:
        op.error('inform either an [input_text] or [input_file] as parameter!')

    horus = Core(False, 5)
    print horus.version_label
    ret = horus.annotate(opts.input_text, opts.input_file, opts.ds_format, opts.output_file, opts.output_format)
    print ret

if __name__ == '__main__':
    main()



import logging


class SystemLog:

    def __init__(self, file, level1, level2):
        logFormatter = logging.Formatter("%(asctime)s [%(threadName)-12.12s] [%(levelname)-5.5s]  %(message)s")
        rootLogger = logging.getLogger()
        fileHandler = logging.FileHandler(file)
        fileHandler.setFormatter(logFormatter)
        fileHandler.setLevel(level1)
        rootLogger.addHandler(fileHandler)
        consoleHandler = logging.StreamHandler()
        consoleHandler.setFormatter(logFormatter)
        rootLogger.addHandler(consoleHandler)
        logging.getLogger("").setLevel(level2)
        self.log = logging



import logging
import urllib2

import urllib3
import urllib
import requests
#sess = requests.Session()
#adapter = requests.adapters.HTTPAdapter(max_retries = 20)
#sess.mount('http://', adapter)
from requests.auth import HTTPBasicAuth
import json

# json_result['d']['results']
# Each result has attributes like 'Description', 'Title', 'Url', '__metadata',
# 'DisplayUrl' and 'ID' so you can do what you want!


def bing_api2(query, api, source_type="Web", top=10, format='json', market='en-US'):

    try:
        keyBing = api  # get Bing key from: https://datamarket.azure.com/account/keys
        credentialBing = 'Basic ' + (':%s' % keyBing).encode('base64')[
                                    :-1]  # the "-1" is to remove the trailing "\n" which encode adds
        query = '%27' + urllib.quote(query) + '%27'
        market = '%27' + urllib.quote(market) + '%27'
        offset = 0

        url = 'https://api.datamarket.azure.com/Bing/Search/' + source_type + \
              '?Query=%s&Market=%s&$top=%d&$skip=%d&$format=json' % (query, market, int(top), offset)

        request = urllib2.Request(url)
        request.add_header('Authorization', credentialBing)
        requestOpener = urllib2.build_opener()
        response = requestOpener.open(request)

        results = json.load(response)

        if response.code != 200:
            return query, None
        else:
            return query, results

    except Exception as e:
        return query, None
        #logging.error(':: an error has occurred: ', e)
        #raise


def bing_api(query, api, source_type="Web", top=10, format='json', market='en-US'):
    """Returns the decoded json response content
    :param query: query for search
    :param source_type: type for seacrh result
    :param top: number of search result
    :param format: format of search result
    :param market: market of search result
    """
    try:
        # set search url
        query = '%27' + urllib.quote(query) + '%27'
        market2 = '%27' + urllib.quote(market) + '%27'
        # web result only base url
        base_url = 'https://api.datamarket.azure.com/Bing/Search/' + source_type
        url = base_url + '?Query=' + query + '&Market=' + market2 + '&$top=' + str(top) + '&$format=' + format

        # create credential for authentication
        user_agent = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_10_3) AppleWebKit/537.36 (KHTML, like Gecko) " \
                     "Chrome/42.0.2311.135 Safari/537.36"

        urllib3.disable_warnings()
        http = urllib3.PoolManager(num_pools=10)
        headers = urllib3.util.make_headers(user_agent=user_agent, basic_auth='{}:{}'.format(api, api))
        resp = http.request('GET',
                            url=url,
                            headers=headers)
        jsonobject = json.loads(resp.data.decode('utf-8'))

        if resp.status != 200:
            return query, None
        else:
            return query, jsonobject

        '''
        # create auth object
        auth = HTTPBasicAuth(api, api)
        # set headers
        headers = {'User-Agent': user_agent}
        # get response from search url
        response_data = sess.get(url, headers=headers, auth=auth)
        # response_data = requests.get(url, headers=headers, auth=auth)

        if response_data.status_code == 200:
            return query, response_data.json()  # decode json response content
        else:
            return query, None
        '''
    except Exception as e:
        logging.error(':: an error has occurred: ', e)
        raise



import os

root = os.getcwd()
'''
-- LOC
'''
PATH_TRAIN_NEG = root + "/img/loc/train/neg/"
PATH_TRAIN_POS1 = root + "/img/loc/train/pos/coast/"
PATH_TRAIN_POS2 = root + "/img/loc/train/pos/forest/"
PATH_TRAIN_POS3 = root + "/img/loc/train/pos/highway/"
PATH_TRAIN_POS4 = root + "/img/loc/train/pos/mountain/"
PATH_TRAIN_POS5 = root + "/img/loc/train/pos/open_country/"
PATH_TRAIN_POS6 = root + "/img/loc/train/pos/street/"
PATH_TRAIN_POS7 = root + "/img/loc/train/pos/suburb/"
PATH_TRAIN_POS8 = root + "/img/loc/train/pos/tall_building/"
PATH_TRAIN_POS9 = root + "/img/loc/train/pos/inside_city/"
PATH_TRAIN_POS10 = root + "/img/loc/train/pos/map2/"


PATH_TEST_NEG = root + "/img/loc/test/neg/"
PATH_TEST_POS1 = root + "/img/loc/test/pos/coast/"
PATH_TEST_POS2 = root + "/img/loc/test/pos/forest/"
PATH_TEST_POS3 = root + "/img/loc/test/pos/highway/"
PATH_TEST_POS4 = root + "/img/loc/test/pos/mountain/"
PATH_TEST_POS5 = root + "/img/loc/test/pos/open_country/"
PATH_TEST_POS6 = root + "/img/loc/test/pos/street/"
PATH_TEST_POS7 = root + "/img/loc/test/pos/suburb/"
PATH_TEST_POS8 = root + "/img/loc/test/pos/tall_building/"
PATH_TEST_POS9 = root + "/img/loc/test/pos/inside_city/"
PATH_TEST_POS10 = root + "/img/loc/test/pos/map2/"

'''
-- ORG
'''
PATH_TRAIN_ORG_POS = root + "/img/org2/train/pos/"
PATH_TRAIN_ORG_NEG = root + "/img/org2/train/neg/"
PATH_TEST_ORG_POS = root + "/img/org2/test/pos/"
PATH_TEST_ORG_NEG = root + "/img/org2/test/neg/"

'''
-- PER
'''
PATH_TRAIN_PER_POS = root + "/img/per/train/pos/"
PATH_TRAIN_PER_NEG = root + "/img/per/train/neg/"
PATH_TEST_PER_POS = root + "/img/per/test/pos/"
PATH_TEST_PER_NEG = root + "/img/per/test/neg/"


def convert_to(path, prefix):
    i = 1
    for filename in os.listdir(path):
        if filename != '.DS_Store':
            newfile = path + prefix + str(i) + filename[-4:]
            os.rename(path + filename, newfile)
            print newfile
            i += 1



#convert_to(PATH_TRAIN_POS1, 'pos-')
#convert_to(PATH_TRAIN_POS2, 'pos-')
#convert_to(PATH_TRAIN_POS3, 'pos-')
#convert_to(PATH_TRAIN_POS4, 'pos-')
#convert_to(PATH_TRAIN_POS5, 'pos-')
#convert_to(PATH_TRAIN_POS6, 'pos-')
#convert_to(PATH_TRAIN_POS7, 'pos-')
#convert_to(PATH_TRAIN_POS8, 'pos-')
#convert_to(PATH_TRAIN_POS9, 'pos-')
#convert_to(PATH_TRAIN_POS10, 'pos-')
convert_to(PATH_TRAIN_NEG, 'neg-')

#convert_to(PATH_TEST_POS1, 'pos-')
#convert_to(PATH_TEST_POS2, 'pos-')
#convert_to(PATH_TEST_POS3, 'pos-')
#convert_to(PATH_TEST_POS4, 'pos-')
#convert_to(PATH_TEST_POS5, 'pos-')
#convert_to(PATH_TEST_POS6, 'pos-')
#convert_to(PATH_TEST_POS7, 'pos-')
#convert_to(PATH_TEST_POS8, 'pos-')
#convert_to(PATH_TEST_POS9, 'pos-')
#convert_to(PATH_TEST_POS10, 'pos-')
#convert_to(PATH_TEST_NEG, 'neg-')

#convert_to(PATH_TRAIN_POS1, 'pos-')


#convert_to(PATH_TRAIN_ORG_POS, 'pos-')
#convert_to(PATH_TRAIN_ORG_NEG, 'neg-')
#convert_to(PATH_TEST_ORG_POS, 'pos-')
#convert_to(PATH_TEST_ORG_NEG, 'neg-')

#convert_to(PATH_TRAIN_PER_POS, 'pos-')
#convert_to(PATH_TRAIN_PER_NEG, 'neg-')
#convert_to(PATH_TEST_PER_POS, 'pos-')
#convert_to(PATH_TEST_PER_NEG, 'neg-')



import os
import cv2
import numpy as np
from os.path import join
from sklearn import svm
from sklearn.externals import joblib


root = os.getcwd()
root_img = root + '/img/'
ner_class = 'loc'
feature = 'map2'

CONST_DATA_POS_PATH      = root_img + ner_class + '/train/pos/' + feature
CONST_DATA_NEG_PATH      = root_img + ner_class + '/train/neg'
CONST_TEST_DATA_POS_PATH = root_img + ner_class + '/test/pos/' + feature + '/'
CONST_TEST_DATA_NEG_PATH = root_img + ner_class + '/test/neg/'

CONST_SVM_MODEL_PATH = root + '/models/' + ner_class + '_' + feature + '_svm2.pkl'
CONST_PREDICTIONS = root + '/predictions/EXP001/' + ner_class + '_' + feature + '_svm_001.txt'
CONST_DICTIONARY_PATH = root + '/models/dict_002_' + ner_class + '_' + feature + '.pkl'
CONST_CREATE_DICTIONARY = 0
CONST_RETRAIN_MODEL = 1


def path(cls,i):
    if cls == "pos-":
        return "%s/%s%d.jpg" % (CONST_DATA_POS_PATH, cls, i+1)
    else:
        return "%s/%s%d.jpg" % (CONST_DATA_NEG_PATH, cls, i+1)

def extract_sift(fn):
    print 'sift: ' + fn
    im = cv2.imread(fn, 0)
    return extract.compute(im, detect.detect(im))[1]

def bow_features(fn):
    print 'bow: ' + fn
    im = cv2.imread(fn, 0)
    return extract_bow.compute(im, detect.detect(im))

def predict(fn):
    f = bow_features(fn); p = svm.predict(f)
    #print fn, "\t", p[1][0][0]
    print fn, "\t", p[0]
    return p

print '- starting the process'
pos, neg = "pos-", "neg-"

detect = cv2.xfeatures2d.SIFT_create()
extract = cv2.xfeatures2d.SIFT_create()

flann_params = dict(algorithm=1, trees=5)
flann = cv2.FlannBasedMatcher(flann_params, {})

bow_kmeans_trainer = cv2.BOWKMeansTrainer(40)
extract_bow = cv2.BOWImgDescriptorExtractor(extract, flann)

if CONST_CREATE_DICTIONARY == 1:
    print '- adding features to cluster'
    for i in range(100): #8
        bow_kmeans_trainer.add(extract_sift(path(pos, i)))
        bow_kmeans_trainer.add(extract_sift(path(neg, i)))

    print '- clustering'
    voc = bow_kmeans_trainer.cluster()
    joblib.dump(voc, CONST_DICTIONARY_PATH, compress=3)
else:
    print '- loading the vocabulary'
    voc = joblib.load(CONST_DICTIONARY_PATH)

print '- setting vocabulary to extractor'
extract_bow.setVocabulary(voc)

if CONST_RETRAIN_MODEL == 1:
    print '- creating training data'
    traindata, trainlabels = [], []
    for i in range(213): # 20
        traindata.extend(bow_features(path(pos, i))); trainlabels.append(1)
        traindata.extend(bow_features(path(neg, i))); trainlabels.append(-1)

    print '- training the model'
    svm = svm.NuSVC(nu=0.5, kernel='rbf', gamma=0.1, probability=True)
    svm.fit(np.array(traindata), np.array(trainlabels))
    joblib.dump(svm, CONST_SVM_MODEL_PATH, compress=3)
else:
    print '- loading the model'
    svm = joblib.load(CONST_SVM_MODEL_PATH)

predictions = []
tot_geral_pos, tot_geral_neg = 0, 0
tot_pos_pos, tot_pos_neg = 0, 0
tot_neg_pos, tot_neg_neg = 0, 0
#pos=0
print '- testing the model (pos) -> ' + CONST_TEST_DATA_POS_PATH
for file in os.listdir(CONST_TEST_DATA_POS_PATH):
    if file != '.DS_Store':
        tot_geral_pos += 1
        obj_predict = predict(CONST_TEST_DATA_POS_PATH + file)
        #if (obj_predict[1][0][0] == 1.0):
        if obj_predict[0] == 1.0:
            tot_pos_pos += 1
        if obj_predict[0] == -1.0:
            tot_pos_neg += 1
        #if 'pos-' in file:
        #    pos = pos + 1
        predictions.append([file, obj_predict[0], '-1'])

print '- testing the model (neg) -> ' + CONST_TEST_DATA_NEG_PATH
for file in os.listdir(CONST_TEST_DATA_NEG_PATH):
    if file != '.DS_Store':
        tot_geral_neg += 1
        obj_predict = predict(CONST_TEST_DATA_NEG_PATH + file)
        #if (obj_predict[1][0][0] == 1.0):
        if obj_predict[0] == 1.0:
            tot_neg_pos += 1
        if obj_predict[0] == -1.0:
            tot_neg_neg += 1
        #if 'pos-' in file:
        #    pos = pos + 1
        predictions.append([file, obj_predict[0], '-1'])

print '- saving predictions...'
predictions = np.array(predictions)
np.savetxt(CONST_PREDICTIONS, predictions, fmt='%s', delimiter=',', newline=os.linesep)

print 'tot files neg = %s | tot files pos = %s' % (tot_geral_neg, tot_geral_pos)
print 'tot files = ' + str(tot_geral_neg + tot_geral_pos)
print 'tot_pos_pos = ' + str(tot_pos_pos) + ' (' + "{0:.2f}".format(float(tot_pos_pos) / float(tot_geral_pos)) + ')'
print 'tot_pos_neg = ' + str(tot_pos_neg) + ' (' + "{0:.2f}".format(float(tot_pos_neg / float(tot_geral_pos))) + ')'
print 'tot_neg_neg = ' + str(tot_neg_neg) + ' (' + "{0:.2f}".format(float(tot_neg_neg) / float(tot_geral_neg)) + ')'
print 'tot_neg_pos = ' + str(tot_neg_pos) + ' (' + "{0:.2f}".format(float(tot_neg_pos / float(tot_geral_neg))) + ')'
acc = float(tot_pos_pos + tot_neg_neg) / float(tot_geral_neg + tot_geral_pos)
print 'acc = ' + str("{0:.2f}".format(acc))
print 'err = ' + str("{0:.2f}".format(float(1 - acc)))



import csv

import numpy as np
import pandas as pd
from sklearn.metrics import accuracy_score
from sklearn.metrics import classification_report
from sklearn.metrics import confusion_matrix
from sklearn.metrics import f1_score
from sklearn.metrics import precision_score
from sklearn.metrics import recall_score

from horus import definitions
from horus.components.config import HorusConfig

config = HorusConfig()
file1reader = csv.reader(open(config.output_path + "experiments/ritter/EXP_000/out_exp000_0.csv"), delimiter=",")
header1 = file1reader.next() #header

tot = 0
tper, torg, tloc = 0, 0, 0

tp_per_cv, tp_org_cv, tp_loc_cv = 0, 0, 0
fp_per_cv, fp_org_cv, fp_loc_cv = 0, 0, 0

tp_per_tx, tp_org_tx, tp_loc_tx = 0, 0, 0
fp_per_tx, fp_org_tx, fp_loc_tx = 0, 0, 0

tp_per, tp_org, tp_loc = 0, 0, 0
fp_per, fp_org, fp_loc = 0, 0, 0

y, ycv, ytx, yf = [], [],[],[]
for linha in file1reader:

    NER = linha[6]
    CV_KLASS = linha[17]
    TX_KLASS = linha[24]
    HORUS_KLASS = linha[25]

    if CV_KLASS == 'LOC':
        ycv.append(1)
    elif CV_KLASS == 'ORG':
        ycv.append(2)
    elif CV_KLASS == 'PER':
        ycv.append(3)
    else:
        ycv.append(4)

    if TX_KLASS == 'LOC':
        ytx.append(1)
    elif TX_KLASS == 'ORG':
        ytx.append(2)
    elif TX_KLASS == 'PER':
        ytx.append(3)
    else:
        ytx.append(4)

    if HORUS_KLASS == 'LOC':
        yf.append(1)
    elif HORUS_KLASS == 'ORG':
        yf.append(2)
    elif HORUS_KLASS == 'PER':
        yf.append(3)
    else:
        yf.append(4)

    if NER in definitions.NER_RITTER_LOC:
        y.append(1)
    elif NER in definitions.NER_RITTER_ORG:
        y.append(2)
    elif NER in definitions.NER_RITTER_PER:
        y.append(3)
    else:
        y.append(4)

    tot+=1
    if NER in definitions.NER_RITTER:
        if NER in definitions.NER_RITTER_PER:
            tper+=1
            if CV_KLASS == 'PER':
                tp_per_cv +=1
            else:
                fp_per_cv +=1
            if TX_KLASS == 'PER':
                tp_per_tx +=1
            else:
                fp_per_tx +=1
            if HORUS_KLASS == 'PER':
                tp_per +=1
            else:
                fp_per +=1
        elif NER in definitions.NER_RITTER_LOC:
            tloc += 1
            if CV_KLASS == 'LOC':
                tp_loc_cv +=1
            else:
                fp_loc_cv +=1
            if TX_KLASS == 'LOC':
                tp_loc_tx +=1
            else:
                fp_loc_tx +=1
            if HORUS_KLASS == 'LOC':
                tp_loc +=1
            else:
                fp_loc +=1
        elif NER in definitions.NER_RITTER_ORG:
            torg += 1
            if CV_KLASS == 'ORG':
                tp_org_cv += 1
            else:
                fp_org_cv += 1
            if TX_KLASS == 'ORG':
                tp_org_tx += 1
            else:
                fp_org_tx += 1
            if HORUS_KLASS == 'ORG':
                tp_org += 1
            else:
                fp_org += 1
    else:
        if CV_KLASS == 'PER':
            fp_per_cv +=1
        if TX_KLASS == 'PER':
            fp_per_tx +=1
        if HORUS_KLASS == 'PER':
            fp_per +=1
        if CV_KLASS == 'ORG':
            fp_org_cv +=1
        if TX_KLASS == 'ORG':
            fp_org_tx +=1
        if HORUS_KLASS == 'ORG':
            fp_org +=1
        if CV_KLASS == 'LOC':
            fp_loc_cv +=1
        if TX_KLASS == 'LOC':
            fp_loc_tx +=1
        if HORUS_KLASS == 'LOC':
            fp_loc +=1

tot_others = tot - (tper+torg+tloc)
print tot, tper, torg, tloc
print tp_per_cv, tp_per_cv/float(tper), tp_org_cv, tp_org_cv/float(torg), tp_loc_cv, tp_loc_cv/float(tloc)
print tp_per_tx, tp_per_tx/float(tper), tp_org_tx, tp_org_tx/float(torg), tp_loc_tx, tp_loc_tx/float(tloc)
print tp_per, tp_per/float(tper), tp_org, tp_org/float(torg), tp_loc, tp_loc/float(tloc)
print '--'
print fp_per_cv, fp_per_cv/float(tot_others), fp_org_cv, fp_org_cv/float(tot_others), fp_loc_cv, fp_loc_cv/float(tot_others)
print fp_per_tx, fp_per_tx/float(tot_others), fp_org_tx, fp_org_tx/float(tot_others), fp_loc_tx, fp_loc_tx/float(tot_others)
print fp_per, fp_per/float(tot_others), fp_org, fp_org/float(tot_others), fp_loc, fp_loc/float(tot_others)


#yt = np.asarray(y).T.tolist()
df = pd.DataFrame(y, columns=["klass"])

print ':: Data Distribution'
p = len(np.where(df["klass"] == 1)[0])
l = len(np.where(df["klass"] == 2)[0])
o = len(np.where(df["klass"] == 3)[0])
nn = len(np.where(df["klass"] == 4)[0])
print '--PER', p, ("%.3f" % (p/float(df.size)))
print '--LOC', l, ("%.3f" % (l/float(df.size)))
print '--ORG', o, ("%.3f" % (o/float(df.size)))
print '--NON', nn, ("%.3f" % (nn/float(df.size)))
print '-----------------------------------------------------'
print ':: Precision (PER, LOC, ORG, OTHERS)'
print '--CV', precision_score(y, ycv, average=None)
print '--TX', precision_score(y, ytx, average=None)
print '--FI', precision_score(y, yf, average=None)
print '-----------------------------------------------------'
print ':: Recall (PER, LOC, ORG, OTHERS)'
print '--CV', recall_score(y, ycv, average=None)
print '--TX', recall_score(y, ytx, average=None)
print '--FI', recall_score(y, yf, average=None)
print '-----------------------------------------------------'
print ':: F-measure (PER, LOC, ORG, OTHERS)'
print '--CV', f1_score(y, ycv, average=None)
print '--TX', f1_score(y, ytx, average=None)
print '--FI', f1_score(y, yf, average=None)
print '-----------------------------------------------------'
print ':: Accuracy (PER, LOC, ORG)'
print '--CV', accuracy_score(y, ycv, normalize=True)
print '--TX', accuracy_score(y, ytx, normalize=True)
print '--FI', accuracy_score(y, yf, normalize=True)
print '-----------------------------------------------------'
print ':: Confusion Matrix'
print '--CV', confusion_matrix(y, ycv)
print '--TX', confusion_matrix(y, ytx)
print '--FI', confusion_matrix(y, yf)
print '-----------------------------------------------------'
target_names = ['PER', 'LOC', 'ORG', 'NON']
print(classification_report(y, ycv, target_names=target_names, digits=3))



import csv

import itertools

import numpy
from sklearn import tree, preprocessing

# "IS_ENTITY?", "ID_SENT", "ID_WORD", "WORD/TERM", "POS_UNI", "POS", "NER", "COMPOUND", "COMPOUND_SIZE", "ID_TERM_TXT",
# "ID_TERM_IMG", "TOT_IMG", "TOT_CV_LOC", "TOT_CV_ORG", "TOT_CV_PER", "DIST_CV_I", "PL_CV_I", "CV_KLASS",
# "TOT_RESULTS_TX", "TOT_TX_LOC", "TOT_TX_ORG", "TOT_TX_PER", "TOT_ERR_TRANS", "DIST_TX_I", "TX_KLASS", "HORUS_KLASS"
from sklearn.cross_validation import train_test_split
from sklearn.feature_extraction import DictVectorizer
from sklearn.metrics import accuracy_score
from sklearn.metrics import confusion_matrix
from sklearn.metrics import f1_score
from sklearn.metrics import precision_score
from sklearn.metrics import recall_score

ner_ritter_per = ['B-person', 'I-person']
ner_ritter_org = ['B-company', 'I-company']
ner_ritter_loc = ['B-geo-loc', 'I-geo-loc']

file1reader = csv.reader(open("/Users/dnes/Github/components-models/components/out_exp000_0.csv"), delimiter=",")
header1 = file1reader.next()

features = []
X, Y = [], []

for linha in file1reader:
    if linha[6] in ner_ritter_per:
        Y.append(1)
    elif linha[6] in ner_ritter_loc:
        Y.append(2)
    elif linha[6] in ner_ritter_org:
        Y.append(3)
    else:
        Y.append(4)

    features.append((linha[5], int(linha[12]), int(linha[13]), int(linha[14]), int(linha[15]),
                    int(linha[16]), int(linha[19]), int(linha[20]), int(linha[21]),
                    float(linha[22]), int(linha[23])))


features = numpy.array(features)
pos = features[:,0]

le = preprocessing.LabelEncoder()
le.fit(pos)

for x in features:
    x[0] = le.transform(x[0])

#vec = DictVectorizer()
#d = dict(itertools.izip_longest(*[iter(features)] * 2, fillvalue=""))
#X = vec.fit_transform([item[0] for item in d]).toarray()

X_train, X_test, Y_train, Y_test = train_test_split(features, Y, train_size=0.50, test_size=0.50)

clf = tree.DecisionTreeClassifier()
clf = clf.fit(X_train, Y_train)
predictions = clf.predict(X_test)

print len(X_train), len(Y_train)
print len(X_test), len(Y_test)

print '--FI', precision_score(Y_test, predictions, average=None)
print '--FI', recall_score(Y_test, predictions, average=None)
print '--FI', f1_score(Y_test, predictions, average=None)
print '--FI', accuracy_score(Y_test, predictions, normalize=True)

print '--FI', confusion_matrix(Y_test, predictions)


#print Y_test
#print predictions

# list(le.inverse_transform([2, 2, 1]))



import os
from horus.components.config import HorusConfig

config = HorusConfig()
if config.root_dir == '':
    ROOT_DIR = os.path.dirname(os.path.abspath(__file__))
else:
    ROOT_DIR = config.root_dir

EXPERIMENTS_PATH = os.path.join(ROOT_DIR, 'experiments')
HORUS_PATH = os.path.join(ROOT_DIR, 'components')
RESOURCES_PATH = os.path.join(ROOT_DIR, 'resource')


NER_RITTER_PER = ['B-person', 'I-person']
NER_RITTER_ORG = ['B-company', 'I-company']
NER_RITTER_LOC = ['B-geo-loc', 'I-geo-loc']

NER_RITTER = []
NER_RITTER.extend(NER_RITTER_PER)
NER_RITTER.extend(NER_RITTER_ORG)
NER_RITTER.extend(NER_RITTER_LOC)

KLASSES = {1: "LOC", 2: "ORG", 3: "PER"}



from distutils.core import setup

setup(name='horusner',
      version='0.1.3',
      description='HORUS Framework',
      author='Diego Esteves',
      author_email='diegoesteves@gmail.com',
      url='http://dnes85.github.io/horus-models/',
      package_dir={'horusner': 'src/horus'},
      packages=['horusner','horusner.components',
                'horusner.experiments', 'horusner.postagger',
                'horusner.resource', 'horusner.sift'],
      )



import os
from ConfigParser import SafeConfigParser
import pkg_resources


class HorusConfig:
    def __init__(self):
        fine = False
        config = None
        for ini_file in os.curdir, os.path.expanduser("~"), "/etc/horus", os.environ.get('HORUS_CONF'):
            try:
                with open(os.path.join(ini_file, "horus.ini")) as source:

                    #config.readfp(source)

                    parser = SafeConfigParser()
                    parser.read(source.name)
                    rootdir = pkg_resources.resource_filename('horus.resource', 'models')

                    self.database_db = parser.get('database', 'db_path')
                    self.root_dir = parser.get('path', 'root_dir')
                    self.output_path = parser.get('path', 'output_path')
                    self.dataset_path = parser.get('path', 'dataset_path')

                    self.models_cv_loc1 = rootdir + parser.get('models-cv', 'horus_loc_1')
                    self.models_cv_loc2 = rootdir + parser.get('models-cv', 'horus_loc_2')
                    self.models_cv_loc3 = rootdir + parser.get('models-cv', 'horus_loc_3')
                    self.models_cv_loc4 = rootdir + parser.get('models-cv', 'horus_loc_4')
                    self.models_cv_loc5 = rootdir + parser.get('models-cv', 'horus_loc_5')
                    self.models_cv_loc6 = rootdir + parser.get('models-cv', 'horus_loc_6')
                    self.models_cv_loc7 = rootdir + parser.get('models-cv', 'horus_loc_7')
                    self.models_cv_loc8 = rootdir + parser.get('models-cv', 'horus_loc_8')
                    self.models_cv_loc9 = rootdir + parser.get('models-cv', 'horus_loc_9')
                    self.models_cv_loc10 = rootdir + parser.get('models-cv', 'horus_loc_10')

                    self.models_cv_loc_1_dict = rootdir + parser.get('models-cv', 'horus_loc_1_voc')
                    self.models_cv_loc_2_dict = rootdir + parser.get('models-cv', 'horus_loc_2_voc')
                    self.models_cv_loc_3_dict = rootdir + parser.get('models-cv', 'horus_loc_3_voc')
                    self.models_cv_loc_4_dict = rootdir + parser.get('models-cv', 'horus_loc_4_voc')
                    self.models_cv_loc_5_dict = rootdir + parser.get('models-cv', 'horus_loc_5_voc')
                    self.models_cv_loc_6_dict = rootdir + parser.get('models-cv', 'horus_loc_6_voc')
                    self.models_cv_loc_7_dict = rootdir + parser.get('models-cv', 'horus_loc_7_voc')
                    self.models_cv_loc_8_dict = rootdir + parser.get('models-cv', 'horus_loc_8_voc')
                    self.models_cv_loc_9_dict = rootdir + parser.get('models-cv', 'horus_loc_9_voc')
                    self.models_cv_loc_10_dict = rootdir + parser.get('models-cv', 'horus_loc_10_voc')

                    self.models_cv_org = rootdir + parser.get('models-cv', 'horus_org')
                    self.models_cv_org_dict = rootdir + parser.get('models-cv', 'horus_org_voc')
                    self.models_cv_per = rootdir + parser.get('models-cv', 'horus_per')
                    self.models_text = rootdir + parser.get('models-text', 'horus_textchecking')

                    self.model_stanford_filename = rootdir + parser.get('model-stanford', 'model_filename')
                    self.model_stanford_path_jar = rootdir + parser.get('model-stanford', 'path_to_jar')

                    self.search_engine_api = parser.get('search-engine', 'api')
                    self.search_engine_key = parser.get('search-engine', 'key')
                    self.search_engine_features_text = parser.get('search-engine', 'features_text')
                    self.search_engine_features_img = parser.get('search-engine', 'features_img')
                    self.search_engine_tot_resources = parser.get('search-engine', 'tot_resources')

                    self.translation_id = parser.get('translation', 'microsoft_client_id')
                    self.translation_secret = parser.get('translation', 'microsoft_client_secret')

                    self.cache_img_folder = parser.get('cache', 'img_folder')

                    self.dataset_ritter = parser.get('dataset', 'ds_ritter')

                    self.models_location_theta = parser.get('models-param', 'location_theta')
                    self.models_distance_theta = parser.get('models-param', 'distance_theta')
                    self.models_safe_interval = parser.get('models-param', 'safe_interval')
                    self.models_limit_min_loc = parser.get('models-param', 'limit_min_loc')
                    self.models_distance_theta_high_bias = parser.get('models-param', 'distance_theta_high_bias')

                    fine = True

                    break
                    #config.readfp(source)
            except IOError:
                pass

        if fine is False:
            raise ValueError('error on trying to read the conf file (horus.conf)! Please set HORUS_CONF with its '
                             'path or place it at your home dir')

        #ini_file = pkg_resources.resource_filename('resource', "horus.conf")
        #rootdir = os.getcwd()
        #

    @staticmethod
    def get_report():
        return 'to be implemented'


import os

import matplotlib.pyplot as plt;

plt.rcdefaults()
import numpy as np
import matplotlib.pyplot as plt

if 1==2:
    objects = ('Coast', 'Forest', 'Highway', 'Mountain', 'Open Country', 'Inside City', 'Street', 'Suburb', 'Tall Building')
    y_pos = np.arange(len(objects))
    performance = [0.89, 0.96, 0.83, 0.84, 0.88, 0.89, 0.87, 0.89, 0.90]

    plt.barh(y_pos, performance, align='center', alpha=0.5, color='red')
    plt.yticks(y_pos, objects)
    plt.xlabel('Accuracy')
    plt.title('Location-based classifiers')

    plt.show()
'''
======================================================= ALL =======================================================
'''

place_acc = (0.89 + 0.96 + 0.83 + 0.84 + 0.88 + 0.89 + 0.87 + 0.89 + 0.90) / 9

place_tp = (0.81 + 1.0 + 0.84 + 0.94 + 0.92 + 0.94 + 0.92 + 0.96 + 0.92) / 9
print place_tp
place_fp = (0.04 + 0.08 + 0.18 + 0.26 + 0.16 + 0.16 + 0.18 + 0.18 + 0.12) / 9
print place_fp
place_fn = (0.19 + 0.00 + 0.16 + 0.06 + 0.08 + 0.06 + 0.08 + 0.04 + 0.08) / 9
print place_fn


# tp / (tp + fp)
p_place = place_tp / (place_tp + place_fp)
# tp / (tp + fn)
r_place = place_tp / (place_tp + place_fn)
# 2 * (precision * recall) / (precision + recall)
f1_place = 2 * (p_place * r_place) / (p_place + r_place)

p_logo = 0.88 / (0.88 + 0.00)
r_logo = 0.88 / (0.88 + 0.12)
f1_logo = 2 * (p_logo * r_logo) / (p_logo + r_logo)

p_face = 0.96 / (0.96 + 0.26)
r_face = 0.96 / (0.96 + 0.04)
f1_face = 2 * (p_face * r_face) / (p_face + r_face)


x = ['Place', 'Logo', 'Face']
y_pos = np.arange(len(x))
y = [f1_place, f1_logo, f1_face]

fig, ax = plt.subplots()
width = 0.50 # the width of the bars
ind = np.arange(len(y))  # the x locations for the groups
ax.barh(ind, y, width, color="blue")
ax.set_yticks(ind+width/2)
ax.set_yticklabels(x, minor=False)
plt.title('HORUS Computer Vision Modules')
plt.xlabel('F1 measure')
plt.ylabel('module')
for i, v in enumerate(y):
    print i, v
    ax.text(v + 0.05, i + .3, "{0:.2f}".format(v), color='blue', fontweight='bold', ha='center', va='center')
#plt.show()
plt.savefig(os.path.join('test.png'), dpi=300, format='png', bbox_inches='tight') # use format='svg' or 'pdf' for vectorial pictures



# -*- coding: utf-8 -*-

"""
==========================================================
HORUS: Named Entity Recognition Algorithm
==========================================================

HORUS is a Named Entity Recognition Algorithm specifically
designed for short-text, i.e., microblogs and other noisy
datasets existing on the web, e.g.: social media, some web-
sites, blogs and etc..

It is a simplistic approach based on multi-level machine
learning combined with computer vision techniques.

more info at: https://github.com/dnes85/components-models

"""

# Author: Esteves <diegoesteves@gmail.com>
# Version: 1.0
# Version Label: HORUS_NER_2016_1.0
# License: BSD 3 clause
import csv
import heapq
import json
import logging
import sqlite3
from time import gmtime, strftime

import cv2
import langdetect
import nltk
import numpy
import requests
from django.core.exceptions import ValidationError
from django.core.validators import URLValidator
from microsofttranslator import Translator
from nltk.tokenize import sent_tokenize
from sklearn.externals import joblib
from sklearn.feature_extraction.text import TfidfTransformer

from bingAPI1 import bing_api2
from config import HorusConfig
from horus import definitions
from horus.components.systemlog import SystemLog

print cv2.__version__


class Core(object):
    """ Description:
            A core module for config.
        Attributes:
            name: A string representing the customer's name.
            balance: A float tracking the current balance of the customer's account.
    """

    # static methods
    version = "0.1.3"
    version_label = "HORUS 0.1.3"

    def __init__(self,force_download,trees):
        """Return a HORUS object"""
        self.sys = SystemLog("horus.log", logging.INFO, logging.INFO)
        self.config = HorusConfig()

        self.sys.log.info('------------------------------------------------------------------')
        self.sys.log.info('::                            HORUS                             ::')
        self.sys.log.info('------------------------------------------------------------------')
        self.sys.log.info(':: loading components...')

        self.english_vocab = None
        self.translator = Translator(self.config.translation_id, self.config.translation_secret)
        self.tfidf_transformer = TfidfTransformer()
        self.detect = cv2.xfeatures2d.SIFT_create()
        self.extract = cv2.xfeatures2d.SIFT_create()
        self.flann_params = dict(algorithm=1, trees=trees)
        self.flann = cv2.FlannBasedMatcher(self.flann_params, {})
        self.extract_bow = cv2.BOWImgDescriptorExtractor(self.extract, self.flann)
        self.svm_logo = joblib.load(self.config.models_cv_org)
        self.voc_org = joblib.load(self.config.models_cv_org_dict)
        self.svm_loc1 = joblib.load(self.config.models_cv_loc1)
        self.svm_loc2 = joblib.load(self.config.models_cv_loc2)
        self.svm_loc3 = joblib.load(self.config.models_cv_loc3)
        self.svm_loc4 = joblib.load(self.config.models_cv_loc4)
        self.svm_loc5 = joblib.load(self.config.models_cv_loc5)
        self.svm_loc6 = joblib.load(self.config.models_cv_loc6)
        self.svm_loc7 = joblib.load(self.config.models_cv_loc7)
        self.svm_loc8 = joblib.load(self.config.models_cv_loc8)
        self.svm_loc9 = joblib.load(self.config.models_cv_loc9)
        self.svm_loc10 = joblib.load(self.config.models_cv_loc10)
        self.voc_loc_1 = joblib.load(self.config.models_cv_loc_1_dict)
        self.voc_loc_2 = joblib.load(self.config.models_cv_loc_2_dict)
        self.voc_loc_3 = joblib.load(self.config.models_cv_loc_3_dict)
        self.voc_loc_4 = joblib.load(self.config.models_cv_loc_4_dict)
        self.voc_loc_5 = joblib.load(self.config.models_cv_loc_5_dict)
        self.voc_loc_6 = joblib.load(self.config.models_cv_loc_6_dict)
        self.voc_loc_7 = joblib.load(self.config.models_cv_loc_7_dict)
        self.voc_loc_8 = joblib.load(self.config.models_cv_loc_8_dict)
        self.voc_loc_9 = joblib.load(self.config.models_cv_loc_9_dict)
        self.voc_loc_10 = joblib.load(self.config.models_cv_loc_10_dict)
        self.text_checking_model = joblib.load(self.config.models_text)
        self.conn = sqlite3.connect(self.config.database_db)
        self.horus_matrix = []
        if force_download is True:
            try:
                nltk.data.find('averaged_perceptron_tagger.zip')
            except LookupError:
                nltk.download('averaged_perceptron_tagger')
            try:
                nltk.data.find('punkt.zip')
            except LookupError:
                nltk.download('punkt')
            try:
                nltk.data.find('maxent_ne_chunker.zip')
            except LookupError:
                nltk.download('maxent_ne_chunker')
            try:
                nltk.data.find('universal_tagset.zip')
            except LookupError:
                nltk.download('universal_tagset')
            try:
                nltk.data.find('words.zip')
            except LookupError:
                nltk.download('words')

    def get_cv_annotation(self):
        x = numpy.array(self.horus_matrix)
        return x[:, [3, 4, 12, 13, 14, 15, 16, 17]]

    def annotate(self, input_text, input_file, ds_format, output_file, output_format):
        # 0 = text (parameter of reading file) / 1 = ritter
        if int(ds_format) == 0:
            text = ''
            if input_text is not None:
                text = input_text.strip('"\'')
                self.sys.log.info(':: processing text')
            elif input_file is not None:
                f = open(input_file, 'r')
                text = f.readlines()
                self.sys.log.info(':: processing input file')
            else:
                raise Exception("err: missing text to be annotated")
            sent_tokenize_list = self.process_input_text(text)

        elif int(ds_format) == 1:  # ritter
            if input_file is None:
                raise Exception("Provide an input file (ritter format) to be annotated")
            else:
                self.sys.log.info(':: loading Ritter ds')
                sent_tokenize_list = self.process_ritter_ds(input_file)

        self.sys.log.info(':: caching %s sentence(s)' % str(len(sent_tokenize_list)))
        # hasEntityNER (1=yes,0=dunno,-1=no), sentence, words[], tags_NER[], tags_POS[], tags_POS_UNI[]
        self.cache_sentence(int(ds_format), sent_tokenize_list)
        self.sys.log.info(':: done!')

        self.sys.log.info(':: caching results...')
        self.cache_results()
        self.sys.log.info(':: done!')

        #  updating components matrix
        # 0 = is_entity?,    1 = index_sent,   2 = index_word, 3 = word/term,
        # 4 = pos_universal, 5 = pos,          6 = ner       , 7 = compound? ,
        # 8 = compound_size, 9 = id_term_txt, 10 = id_term_img
        self.sys.log.info(':: detecting %s objects...' % len(self.horus_matrix))
        self.detect_objects()
        self.sys.log.info(':: done!')

        self.conn.close()

        self.sys.log.info(':: applying rules...')
        self.update_rules_cv_predictions()
        self.sys.log.info(':: updating compounds...')
        self.update_compound_predictions()
        self.sys.log.info(':: done!')

        # self.sys.log.info(horus_matrix)
        header = ["IS_ENTITY?", "ID_SENT", "ID_WORD", "WORD/TERM", "POS_UNI", "POS", "NER", "COMPOUND", "COMPOUND_SIZE",
                  "ID_TERM_TXT", "ID_TERM_IMG",
                  "TOT_IMG", "TOT_CV_LOC", "TOT_CV_ORG", "TOT_CV_PER", "DIST_CV_I", "PL_CV_I", "CV_KLASS", "TOT_RESULTS_TX",
                  "TOT_TX_LOC", "TOT_TX_ORG",
                  "TOT_TX_PER", "TOT_ERR_TRANS", "DIST_TX_I", "TX_KLASS", "HORUS_KLASS"]

        if int(ds_format) == 0:
            self.print_annotated_sentence()

        self.sys.log.info(':: exporting metadata...')
        if output_file == '':
            output_file = 'noname'
        if output_format == 'json':
            with open(output_file + '.json', 'wb') as outfile:
                json.dump(self.horus_matrix, outfile)
        elif output_format == 'csv':
            horus_csv = open(output_file + '.csv', 'wb')
            wr = csv.writer(horus_csv, quoting=csv.QUOTE_ALL)
            wr.writerow(header)
            wr.writerows(self.horus_matrix)

        return self.horus_matrix

    def print_annotated_sentence(self):
        '''
        read the components matrix and prints the annotated sentences
        :param horus_matrix:
        :return: output of annotated sentence
        '''
        x = ''
        id_sent_aux = self.horus_matrix[0][1]
        for term in self.horus_matrix:
            if term[7] == 0:
                if id_sent_aux != term[1]:
                    #self.sys.log.info(':: sentence ' + str(id_sent_aux) + ': ' + x)
                    self.sys.log.info(':: sentence: ' + x)
                    id_sent_aux = term[1]
                    x = ' ' + str(term[3]) + '/' + str(term[25])
                else:
                    x += ' ' + str(term[3]) + '/' + str(term[4]) + '/' + str(term[5]) + '/' + str(term[25])
                    # x += ' ' + str(term[3]) + '/' + str(term[25])

        self.sys.log.info(':: sentence: ' + x)

    def cache_sentence_ritter(self,sentence_list):
        self.sys.log.debug(':: caching Ritter dataset...:')
        i_sent, i_word = 1, 1
        compound, prev_tag = '', ''
        sent_with_ner = 0
        token_ok = 0
        compound_ok = 0
        for sent in sentence_list:

            self.sys.log.debug(':: processing sentence: ' + sent[1])

            # processing compounds
            if sent[0] == 1:
                sent_with_ner += 1
                for tag in sent[3]:  # list of NER tags
                    word = sent[2][i_word - 1]
                    if tag in definitions.NER_RITTER:  # only desired tags
                        if prev_tag.replace('B-', '').replace('I-', '') == tag.replace('B-', '').replace('I-', ''):
                            compound += prev_word + ' ' + word + ' '
                    prev_word = word
                    prev_tag = tag
                    i_word += 1
                compound = compound[:-1]

                if compound != '':
                    compound_ok+=1
                    self.horus_matrix.append([1, i_sent, i_word - len(compound.split(' ')), compound, '', '', '', 1, len(compound.split(' '))])
                    compound = ''
                prev_tag = ''
                prev_word = ''

            # processing tokens

            #  transforming to components matrix
            # 0 = is_entity?,    1 = index_sent, 2 = index_word, 3 = word/term,
            # 4 = pos_universal, 5 = pos,        6 = ner       , 7 = compound? ,
            # 8 = compound_size

            i_word = 1
            for k in range(len(sent[2])): # list of NER tags
                is_entity = 1 if sent[3] in definitions.NER_RITTER else 0
                self.horus_matrix.append([is_entity, i_sent, i_word, sent[2][k], sent[5][k], sent[4][k], sent[3][k], 0, 0])
                i_word += 1
                if is_entity:
                    token_ok += 1

            self.db_save_sentence(sent[1], '-', '-', str(sent[3]))
            i_sent += 1
            i_word = 1

        self.sys.log.debug(':: done! total of sentences = %s, tokens = %s and compounds = %s'
                     % (str(sent_with_ner), str(token_ok), str(compound_ok)))


    def tokenize_and_pos(self,sentence, tagset=''):
        self.sys.log.debug(':: processing sentence: ' + sentence)
        tokens = nltk.word_tokenize(sentence)
        self.sys.log.debug('---------- ' + str(tokens))
        tagged = nltk.pos_tag(tokens, tagset=tagset)
        return tokens, tagged

    def cache_sentence(self,sentence_format, sentence_list):
        if sentence_format == 0:
            self.cache_sentence_free_text(sentence_list)
        elif sentence_format == 1:
            self.cache_sentence_ritter(sentence_list)

    def cache_sentence_free_text(self,sentence_list):

        i_sent, i_word = 1, 1
        self.sys.log.debug(':: chunking pattern ...')
        pattern = "NP:{<NN|NNP|NNS|NNPS>+}"
        cp = nltk.RegexpParser(pattern)
        compounds = '|'

        for sent in sentence_list:
            #tokens, tagged = tokenize_and_pos(sent[1])
            #self.sys.log.info(':: tags: ' + str(tagged))
            ## entities = nltk.chunk.ne_chunk(tagged)

            #  add compounds of given sentence

            aux = 0
            toparse = []
            for obj in sent[2]:
                toparse.append(tuple([obj, sent[4][aux]]))
                aux+=1
            t = cp.parse(toparse)
            i_word = 1
            for item in t:
                is_entity = 1 if (sent[0] == 1 and sent[3][i_word - 1] != 'O') else -1
                if type(item) is nltk.Tree:  # that's a compound
                    compound = ''
                    for tk in item:
                        compound += tk[0] + ' '
                        i_word += 1
                    if len(item) > 1:
                        self.horus_matrix.append([is_entity, i_sent, i_word - len(item), compound[:len(compound) - 1], '', '', '', 1, len(item)])
                        compounds += compound[:len(compound) - 1] + '|'
                    compound = ''

            #  transforming to components matrix
            # 0 = is_entity?,    1 = index_sent, 2 = index_word, 3 = word/term,
            # 4 = pos_universal, 5 = pos,        6 = ner       , 7 = compound? ,
            # 8 = compound_size
            i_word = 1
            for k in range(len(sent[2])):
                is_entity = 1 if (sent[0] == 1 and sent[3][k] != 'O') else -1
                self.horus_matrix.append([is_entity, i_sent, i_word, sent[2][k], sent[5][k], sent[4][k], sent[3][k], 0, 0])
                i_word += 1

            self.db_save_sentence(sent[1], ' '.join(sent[5]), compounds, ' '.join(sent[2]))

            i_sent += 1

    def db_save_sentence(self,sent, tagged, compound, tokens):
        c = self.conn.cursor()
        self.conn.text_factory = str
        sql = """SELECT id FROM HORUS_SENTENCES WHERE sentence = ? """
        c.execute(sql, (sent,))
        res_sent = c.fetchone()
        if res_sent is None:
            self.sys.log.info(':: caching sentence: ' + sent)
            #buffer(zlib.compress
            row = (str(sent), str(tagged), str(compound), str(tokens))
            sql = """INSERT INTO HORUS_SENTENCES(sentence, tagged, compounds, tokens)
                             VALUES(?,?,?,?)"""
            self.sys.log.debug(sql)
            c.execute(sql, row)
            self.conn.commit()
            self.sys.log.debug(':: done: ' + sent)
        else:
            self.sys.log.debug(':: sentence is already cached')

    def download_image_local(self,image_url, image_type, thumbs_url, thumbs_type, term_id, id_ner_type, seq):
        val = URLValidator()
        auxtype = None
        try:
            val(thumbs_url)
            try:
                img_data = requests.get(thumbs_url).content
                with open('%s%s_%s_%s.%s' % (self.config.cache_img_folder, term_id, id_ner_type, seq, thumbs_type.split('/')[1]), 'wb') as handler:
                    handler.write(img_data)
                    auxtype = thumbs_type.split('/')[1]
            except Exception as error:
                print('-> error: ' + repr(error))
        except ValidationError, e:
            self.sys.log.error('No thumbs img here...', e)
            try:
                img_data = requests.get(image_url).content
                with open('%s%s_%s_%s.%s' % (self.config.cache_img_folder, term_id, id_ner_type, seq, image_type.split('/')[1]), 'wb') as handler:
                    auxtype = image_type.split('/')[1]
                    handler.write(img_data)
            except Exception as error:
                print('-> error: ' + repr(error))
        return auxtype

    def cache_results(self):
        try:
            auxc = 1
            self.sys.log.info(':: caching horus_matrix: ' + str(len(self.horus_matrix)))

            for item in self.horus_matrix:
                self.sys.log.info(':: item %s - %s ' % (str(auxc), str(len(self.horus_matrix))))
                term = item[3]
                if item[4] == 'NOUN' or item[7] == 1:
                    self.sys.log.debug(':: caching [%s] ...' % term)
                    c = self.conn.cursor()

                    # checking if we have searched that before (might be the case of had used
                    # different configurations, or different search engine, for instance.
                    sql = """SELECT id FROM HORUS_TERM WHERE term = ?"""
                    c.execute(sql, (term,))
                    res_term = c.fetchone()
                    if res_term is None:
                        self.sys.log.info(':: [%s] has not been cached before!' % term)
                        cterm = self.conn.execute("""INSERT INTO HORUS_TERM(term) VALUES(?)""", (term,))
                    else:
                        cterm = res_term[0]

                    # check if term (text) has been cached before
                    # in case components is extended to accept more than 1 search engine, this table should also
                    # have it defined
                    values = (term, self.config.search_engine_api, 1, self.config.search_engine_features_text)
                    sql = """SELECT id
                             FROM HORUS_TERM_SEARCH
                             WHERE term = ? AND
                                   id_search_engine = ? AND
                                   id_search_type = ? AND
                                   search_engine_features = ?"""
                    c.execute(sql, values)
                    res = c.fetchone()
                    if res is None:
                        self.sys.log.info(':: [%s] caching - text' % term)
                        values = (term, cterm.lastrowid, self.config.search_engine_api, 1,
                                  self.config.search_engine_features_text,
                                  str(strftime("%Y-%m-%d %H:%M:%S", gmtime())),
                                  self.config.search_engine_tot_resources)
                        c = self.conn.execute("""INSERT into HORUS_TERM_SEARCH(term, id_term, id_search_engine, id_search_type,
                                                                          search_engine_features, query_date,
                                                                          query_tot_resource)
                                                 VALUES(?, ?, ?, ?, ?, ?, ?)""", values)

                        id_term_search = c.lastrowid
                        item.extend([id_term_search])  # updating matrix
                        seq = 0
                        # get text
                        metaquery, result = bing_api2(term, api=self.config.search_engine_key, source_type="Web",
                                                     top=self.config.search_engine_tot_resources, format='json', market='en-US')
                        for web_result in result['d']['results']:
                            seq+=1
                            row = (id_term_search,
                                   0,
                                   web_result['ID'],
                                   seq,
                                   web_result['Url'],
                                   web_result['Title'],
                                   web_result['Description'],
                                   '')
                            c.execute("""INSERT INTO HORUS_SEARCH_RESULT_TEXT (id_term_search,
                                                                                    id_ner_type,
                                                                                    search_engine_resource_id,
                                                                                    result_seq,
                                                                                    result_url,
                                                                                    result_title,
                                                                                    result_description,
                                                                                    result_html_text)
                                              VALUES(?,?,?,?,?,?,?,?)""", row)

                            c.execute("""UPDATE HORUS_TERM_SEARCH
                                              SET metaquery = '%s'
                                              WHERE id = %s""" % (metaquery, id_term_search))

                        if seq == 0:
                            row = (id_term_search,
                                   0,
                                   '',
                                   seq,
                                   '',
                                   '',
                                   '',
                                   '')
                            c.execute("""INSERT INTO HORUS_SEARCH_RESULT_TEXT (id_term_search,
                                                                               id_ner_type,
                                                                               search_engine_resource_id,
                                                                               result_seq,
                                                                               result_url,
                                                                               result_title,
                                                                               result_description,
                                                                               result_html_text)
                                                                      VALUES(?,?,?,?,?,?,?,?)""", row)

                            c.execute("""UPDATE HORUS_TERM_SEARCH
                                         SET metaquery = '%s'
                                         WHERE id = %s""" % (metaquery, id_term_search))

                        self.sys.log.debug(':: term [%s] cached (text)!' % term)
                        self.conn.commit()
                    else:
                        item.extend(res)  # updating matrix
                        self.sys.log.debug(':: term %s is already cached (text)!' % term)

                    values = (term, self.config.search_engine_api, 2, self.config.search_engine_features_text)
                    c = self.conn.execute("""SELECT id
                                        FROM HORUS_TERM_SEARCH
                                        WHERE term = ? AND
                                              id_search_engine = ? AND
                                              id_search_type = ? AND
                                              search_engine_features = ?""", values)
                    res = c.fetchone()
                    if res is None:
                        self.sys.log.info(':: [%s] caching - image' % term)
                        values = (term, cterm.lastrowid if type(cterm) is not int else cterm, self.config.search_engine_api, 2,
                                   self.config.search_engine_features_img,
                                   str(strftime("%Y-%m-%d %H:%M:%S", gmtime())), self.config.search_engine_tot_resources)
                        sql = """INSERT into HORUS_TERM_SEARCH(term, id_term, id_search_engine, id_search_type,
                                                               search_engine_features, query_date, query_tot_resource)
                                                 VALUES(?,?,?,?,?,?,?)"""
                        c.execute(sql, values)
                        id_term_img = c.lastrowid
                        item.extend([id_term_img])  # updating matrix
                        seq = 0
                        # get images
                        metaquery, result = bing_api2(item[3], api=self.config.search_engine_key, source_type="Image",
                                                     top=self.config.search_engine_tot_resources, format='json')
                        for web_img_result in result['d']['results']:
                            self.sys.log.debug(':: downloading image [%s]' % (web_img_result['Title']))
                            seq += 1
                            auxtype = self.download_image_local(web_img_result['MediaUrl'],
                                                 web_img_result['ContentType'],
                                                 web_img_result['Thumbnail']['MediaUrl'],
                                                 web_img_result['Thumbnail']['ContentType'],
                                                 id_term_img,
                                                 0,
                                                 seq)
                            self.sys.log.debug(':: caching image result ...')
                            fname = ('%s_%s_%s.%s' % (str(id_term_img),  str(0),  str(seq),  str(auxtype)))
                            row = (id_term_img,
                                   0,
                                   web_img_result['ID'],
                                   seq,
                                   web_img_result['MediaUrl'],
                                   web_img_result['Title'],
                                   web_img_result['ContentType'],
                                   web_img_result['Height'],
                                   web_img_result['Width'],
                                   web_img_result['Thumbnail']['MediaUrl'],
                                   web_img_result['Thumbnail']['ContentType'],
                                   fname)

                            sql = """INSERT INTO HORUS_SEARCH_RESULT_IMG (id_term_search,
                                                                          id_ner_type,
                                                                          search_engine_resource_id,
                                                                          result_seq,
                                                                          result_media_url,
                                                                          result_media_title,
                                                                          result_media_content_type,
                                                                          result_media_height,
                                                                          result_media_width,
                                                                          result_media_thumb_media_url,
                                                                          result_media_thumb_media_content_type,
                                                                          filename)
                                              VALUES(?,?,?,?,?,?,?,?,?,?,?,?)"""
                            c.execute(sql, row)

                            c.execute("""UPDATE HORUS_TERM_SEARCH
                                              SET metaquery = '%s'
                                              WHERE id = %s""" % (metaquery, id_term_img))

                        self.sys.log.debug(':: term [%s] cached (img)!' % term)
                        self.conn.commit()
                    else:
                        self.sys.log.debug(':: term %s is already cached (img)!' % term)
                        item.extend(res)  # updating matrix

                auxc+=1
        except Exception as e:
            self.sys.log.error(':: an error has occurred: ', e)
            raise

    def detect_faces(self,img):
        try:
            # print cv2.__version__
            face_cascade = cv2.CascadeClassifier(self.config.models_cv_per)
            image = cv2.imread(img)
            if image is None:
                self.sys.log.error('could not load the image: ' + img)
                return -1
            gray = cv2.cvtColor(image, cv2.COLOR_BGR2GRAY)
            faces = face_cascade.detectMultiScale(gray, scaleFactor=1.3, minNeighbors=5, flags=cv2.CASCADE_SCALE_IMAGE)
            return len(faces)
            # cv2.CV_HAAR_SCALE_IMAGE #
            # minSize=(30, 30)

            ## Draw a rectangle around the faces
            # for (x, y, w, h) in faces:
            #    cv2.rectangle(image, (x, y), (x + w, y + h), (0, 255, 0), 2)
            # cv2.imshow("Faces found", image)
            # cv2.waitKey(0)

        except Exception as error:
            self.sys.log.error(':: error: ' + repr(error))
            return -1

    def bow_features(self,fn, ner_type):
        im = cv2.imread(fn, 0)
        if ner_type == 'ORG_1':
            self.extract_bow.setVocabulary(self.voc_org)
        elif ner_type == 'LOC_1':
            self.extract_bow.setVocabulary(self.voc_loc_1)
        elif ner_type == 'LOC_2':
            self.extract_bow.setVocabulary(self.voc_loc_2)
        elif ner_type == 'LOC_3':
            self.extract_bow.setVocabulary(self.voc_loc_3)
        elif ner_type == 'LOC_4':
            self.extract_bow.setVocabulary(self.voc_loc_4)
        elif ner_type == 'LOC_5':
            self.extract_bow.setVocabulary(self.voc_loc_5)
        elif ner_type == 'LOC_6':
            self.extract_bow.setVocabulary(self.voc_loc_6)
        elif ner_type == 'LOC_7':
            self.extract_bow.setVocabulary(self.voc_loc_7)
        elif ner_type == 'LOC_8':
            self.extract_bow.setVocabulary(self.voc_loc_8)
        elif ner_type == 'LOC_9':
            self.extract_bow.setVocabulary(self.voc_loc_9)
        elif ner_type == 'LOC_10':
            self.extract_bow.setVocabulary(self.voc_loc_10)

        return self.extract_bow.compute(im, self.detect.detect(im))

    def detect_logo(self,img):
        f = self.bow_features(img, 'ORG_1');
        if f is None:
            self.sys.log.warn(':: feature extraction error!')
            p = [0]
        else:
            p = self.svm_logo.predict(f)
        self.sys.log.debug('predicted class -> ' + str(p))
        return p

    def detect_place(self,img):
        self.sys.log.debug(':: detecting places...')
        ret = []
        f = self.bow_features(img, 'LOC_1');
        if f is None:
            self.sys.log.warn(':: feature extraction error!')
            ret.append(-1)
        else:
            ret.append(self.svm_loc1.predict(f)[0])

        f = self.bow_features(img, 'LOC_2');
        if f is None:
            self.sys.log.warn(':: feature extraction error!')
            ret.append(-1)
        else:
            ret.append(self.svm_loc2.predict(f)[0])

        f = self.bow_features(img, 'LOC_3');
        if f is None:
            self.sys.log.warn(':: feature extraction error!')
            ret.append(-1)
        else:
            ret.append(self.svm_loc3.predict(f)[0])

        f = self.bow_features(img, 'LOC_4');
        if f is None:
            self.sys.log.warn(':: feature extraction error!')
            ret.append(-1)
        else:
            ret.append(self.svm_loc4.predict(f)[0])

        f = self.bow_features(img, 'LOC_5');
        if f is None:
            self.sys.log.warn(':: feature extraction error!')
            ret.append(-1)
        else:
            ret.append(self.svm_loc5.predict(f)[0])

        f = self.bow_features(img, 'LOC_6');
        if f is None:
            self.sys.log.warn(':: feature extraction error!')
            ret.append(-1)
        else:
            ret.append(self.svm_loc6.predict(f)[0])

        f = self.bow_features(img, 'LOC_7');
        if f is None:
            self.sys.log.warn(':: feature extraction error!')
            ret.append(-1)
        else:
            ret.append(self.svm_loc7.predict(f)[0])

        f = self.bow_features(img, 'LOC_8');
        if f is None:
            self.sys.log.warn(':: feature extraction error!')
            ret.append(-1)
        else:
            ret.append(self.svm_loc8.predict(f)[0])

        f = self.bow_features(img, 'LOC_9');
        if f is None:
            self.sys.log.warn(':: feature extraction error!')
            ret.append(-1)
        else:
            ret.append(self.svm_loc9.predict(f)[0])

        f = self.bow_features(img, 'LOC_10');
        if f is None:
            self.sys.log.warn(':: feature extraction error!')
            ret.append(-1)
        else:
            ret.append(self.svm_loc10.predict(f)[0])

        return ret

    def detect_text_klass(self,t1, t2, id, t1en, t2en):

        self.sys.log.debug(':: text analysis component launched')

        try:
            from translate import Translator

            #https://pypi.python.org/pypi/translate (alternative 1000 per day)
            #https://www.microsoft.com/en-us/translator/getstarted.aspx
            #https://github.com/openlabs/Microsoft-Translator-Python-API

            c = self.conn.cursor()
            t1final = t1
            t2final = t2

            # need to save to components db
            if t1en is None:
                lt1 = langdetect.detect(t1)
                if lt1 != 'en':
                    try:
                        t1final = self.translator.translate(t1, 'en')
                    except Exception as e1:
                        self.sys.log.error(':: Error, trying another service: ' + str(e1))
                        try:
                            translator2 = Translator(from_lang=lt1, to_lang="en")
                            t1final = translator2.translate(t1)
                        except Exception as e2:
                            self.sys.log.error(':: Error at service 2: ' + str(e2))
                            return [-1]
                            # updating

                t1final = 'u'+t1final # .encode('ascii', 'ignore')
                sql = """UPDATE HORUS_SEARCH_RESULT_TEXT
                         SET result_title_en = ? WHERE id = ?"""
                c.execute(sql, (t1final, id))
                self.conn.commit()
            else:
                t1final = t1en

            if t2en is None:
                lt2 = langdetect.detect(t2)
                if lt2 != 'en':
                    try:
                        t2final = self.translator.translate(t2, 'en')
                    except Exception as e1:
                        self.sys.log.error(':: Error, trying another service: ' + str(e1))
                        try:
                            translator2 = Translator(from_lang=lt2, to_lang="en")
                            t2final = translator2.translate(t2)
                        except Exception as e2:
                            self.sys.log.error(':: Error at service 2: ' + str(e2))
                            return [-1]
                            # updating

                #t2final = t2final.encode('ascii', 'ignore')
                t2final = 'u'+t2final
                sql = """UPDATE HORUS_SEARCH_RESULT_TEXT
                                SET result_description_en = ? WHERE id = ?"""
                c.execute(sql, (t2final, id))
                self.conn.commit()
            else:
                t2final = t2en

            c.close()
            docs = ["{} {}".format(t1final, t2final)]
            predicted = self.text_checking_model.predict(docs)
            return predicted

            #blob = TextBlob(t2)
            #t22 = blob.translate(to='en')
            # text_vocab = set(w.lower() for w in t2 if w.lower().isalpha())
            # unusual = text_vocab.difference(english_vocab)

        except Exception as e:
            self.sys.log.error(':: Error: ' + str(e))
            return [-1]

    def detect_objects(self):     # id_term_img, id_term_txt, id_ner_type, term
        auxi = 0
        toti = len(self.horus_matrix)
        for item in self.horus_matrix:
            auxi += 1
            if item[4] == 'NOUN' or item[7] == 1:

                self.sys.log.info(':: processing item %d of %d' % (auxi, toti))

                id_term_img = item[10]
                id_term_txt = item[9]
                id_ner_type = 0
                term = item[3]

                tot_geral_faces = 0
                tot_geral_logos = 0
                tot_geral_locations = 0
                tot_geral_pos_locations = 0
                tot_geral_neg_locations = 0
                T = int(self.config.models_location_theta)  # location threshold

                filesimg = []
                metadata = []
                with self.conn:
                    cursor = self.conn.cursor()
                    cursor.execute("""SELECT filename,
                                                 id,
                                                 processed,
                                                 nr_faces,
                                                 nr_logos,
                                                 nr_place_1,
                                                 nr_place_2,
                                                 nr_place_3,
                                                 nr_place_4,
                                                 nr_place_5,
                                                 nr_place_6,
                                                 nr_place_7,
                                                 nr_place_8,
                                                 nr_place_9,
                                                 nr_place_10
                                          FROM HORUS_SEARCH_RESULT_IMG
                                          WHERE id_term_search = %s AND id_ner_type = %s""" % (id_term_img, id_ner_type))
                    rows = cursor.fetchall()
                    tot_img = len(rows)

                    for row in rows:  # 0 = file path | 1 = id | 2 = processed | 3=nr_faces | 4=nr_logos | 5 to 13=nr_places_1 to 9
                        filesimg.append((self.config.cache_img_folder + row[0],
                                         row[1],
                                         row[2],
                                         row[3],
                                         row[4],
                                         row[5], row[6], row[7], row[8], row[9], row[10], row[11], row[12], row[13],
                                         row[14]))

                for image_term in filesimg:
                    if image_term[2] == 1:
                        tot_geral_faces += image_term[3]
                        tot_geral_logos += image_term[4]
                        if (image_term[5:13]).count(1) >= int(T):
                            tot_geral_locations += 1
                        tot_geral_pos_locations += image_term[5:13].count(1)
                        tot_geral_neg_locations += (image_term[5:13].count(-1) * -1)
                    else:
                        # ----- face recognition -----
                        tot_faces = self.detect_faces(image_term[0])
                        if tot_faces > 0:
                            tot_geral_faces += 1
                            self.sys.log.debug(":: found {0} faces!".format(tot_faces))

                        # ----- logo recognition -----
                        tot_logos = self.detect_logo(image_term[0])
                        if tot_logos[0] == 1:
                            tot_geral_logos += 1
                            self.sys.log.debug(":: found {0} logo(s)!".format(1))

                        # ----- place recognition -----
                        res = self.detect_place(image_term[0])
                        tot_geral_pos_locations += res.count(1)
                        tot_geral_neg_locations += (res.count(-1) * -1)

                        if res.count(1) >= T:
                            tot_geral_locations += 1
                            self.sys.log.debug(":: found {0} place(s)!".format(1))

                        # updating results
                        sql = """UPDATE HORUS_SEARCH_RESULT_IMG
                                     SET nr_faces = ?, nr_logos = ?, nr_place_1 = ?, nr_place_2 = ?, nr_place_3 = ?,
                                         nr_place_4 = ?, nr_place_5 = ?, nr_place_6 = ?, nr_place_7 = ?, nr_place_8 = ?,
                                         nr_place_9 = ?, nr_place_10 = ?, processed = 1
                                     WHERE id = ?"""
                        param = []
                        param.append(tot_faces)
                        param.append(tot_logos[0]) if tot_logos[0] == 1 else param.append(0)
                        param.extend(res)
                        param.append(image_term[1])
                        cursor.execute(sql, param)

                self.conn.commit()

                outs = [tot_geral_locations, tot_geral_logos, tot_geral_faces]
                maxs_cv = heapq.nlargest(2, outs)
                dist_cv_indicator = max(maxs_cv) - min(maxs_cv)
                place_cv_indicator = tot_geral_pos_locations + tot_geral_neg_locations

                # 0 to 5
                metadata.append(tot_img)
                metadata.append(tot_geral_locations)  # 1
                metadata.append(tot_geral_logos)  # 2
                metadata.append(tot_geral_faces)  # 3
                metadata.append(dist_cv_indicator)  # 4
                metadata.append(place_cv_indicator)  # 5

                self.sys.log.debug('-------------------------------------------------------------')
                self.sys.log.debug(':: [checking related visual information for [%s]]' % term)
                self.sys.log.debug('')
                self.sys.log.debug('-> CV_LOC  indicator: %f %%' % (float(tot_geral_locations) / tot_img)) if tot_img > 0 \
                    else self.sys.log.debug('-> CV_LOC  indicator: err no img retrieved')
                self.sys.log.debug('-> CV_ORG  indicator: %f %%' % (float(tot_geral_logos) / tot_img)) if tot_img > 0 \
                    else self.sys.log.debug('-> CV_ORG  indicator: err no img retrieved')
                self.sys.log.debug('-> CV_PER  indicator: %f %%' % (float(tot_geral_faces) / tot_img)) if tot_img > 0 \
                    else self.sys.log.debug('-> CV_PER  indicator: err no img retrieved')
                self.sys.log.debug('-> CV_DIST indicator: %s' % (str(dist_cv_indicator)))
                self.sys.log.debug('-> CV_PLC  indicator: %s' % (str(place_cv_indicator)))

                metadata.append(definitions.KLASSES[outs.index(max(outs)) + 1])

                # text classification
                self.sys.log.debug(':: [checking related textual information ...]')
                y = []
                with self.conn:
                    cursor = self.conn.cursor()
                    cursor.execute("""SELECT id, result_seq, result_title, result_description, result_title_en,
                                                 result_description_en, processed, text_klass
                                          FROM HORUS_SEARCH_RESULT_TEXT
                                          WHERE id_term_search = %s AND id_ner_type = %s""" % (id_term_txt, id_ner_type))
                    rows = cursor.fetchall()
                    tot_err = 0
                    for row in rows:
                        if row[6] == 0 or row[6] is None:
                            ret = self.detect_text_klass(row[2], row[3], row[0], row[4], row[5])
                            if ret[0] != -1:
                                y.append(ret)
                                sql = """UPDATE HORUS_SEARCH_RESULT_TEXT SET text_klass = %s , processed = 1
                                         WHERE id = %s""" % (ret[0], row[0])
                                self.sys.log.debug(':: ' + sql)
                                cursor.execute(sql)
                            else:
                                tot_err += 1
                        else:
                            y.append(row[7])

                    self.conn.commit()

                    gp = [y.count(1), y.count(2), y.count(3)]
                    horus_tx_ner = gp.index(max(gp)) + 1

                    self.sys.log.debug(':: final textual checking statistics for term [%s] '
                                    '(1-LOC = %s, 2-ORG = %s and 3-PER = %s)' % (term, str(y.count(1)), str(y.count(2)),
                                                                                 str(y.count(3))))
                    # 7 to 11
                    metadata.append(len(rows))
                    metadata.append(y.count(1))
                    metadata.append(y.count(2))
                    metadata.append(y.count(3))
                    metadata.append(float(tot_err))

                    maxs_tx = heapq.nlargest(2, gp)
                    dist_tx_indicator = max(maxs_tx) - min(maxs_tx)

                    # 12, 13
                    metadata.append(dist_tx_indicator)
                    metadata.append(definitions.KLASSES[horus_tx_ner])

                    if len(rows) != 0:
                        self.sys.log.debug('-> TX_LOC  indicator: %f %%' % (float(y.count(1)) / len(rows)))
                        self.sys.log.debug('-> TX_ORG  indicator: %f %%' % (float(y.count(2)) / len(rows)))
                        self.sys.log.debug('-> TX_PER  indicator: %f %%' % (float(y.count(3)) / len(rows)))
                        self.sys.log.debug('-> TX_DIST indicator: %s' % (str(dist_tx_indicator)))
                        self.sys.log.debug(':: number of trans. errors -> ' + str(tot_err) + ' over ' + str(len(rows)))
                        self.sys.log.debug(':: most likely class -> ' + definitions.KLASSES[horus_tx_ner])
                    else:
                        self.sys.log.debug(':: there was a problem searching this term..please try to index it again...')

                    # checking final NER - 14
                    if metadata[4] >= int(self.config.models_distance_theta):
                        metadata.append(metadata[6])  # CV is the final decision
                    else:
                        metadata.append(metadata[13])  # TX is the final decision

                item.extend(metadata)
            else:
                item.extend([0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0])

    def update_rules_cv_predictions(self):
        '''
        updates the predictions based on inner rules
        :return:
        '''
        self.sys.log.info(':: updating predictions based on rules')
        for i in range(len(self.horus_matrix)):
            initial = self.horus_matrix[i][17]
            # get nouns or compounds
            if self.horus_matrix[i][4] == u'NOUN' or self.horus_matrix[i][7] == 1:
                # do not consider classifications below a theta
                if self.horus_matrix[i][15] < int(self.config.models_distance_theta):
                    self.horus_matrix[i][17] = "*"
                # ignore LOC classes having iPLC negative
                if bool(self.config.models_distance_theta_high_bias) is True:
                    if initial == "LOC":
                        if self.horus_matrix[i][16] < int(self.config.models_limit_min_loc):
                            self.horus_matrix[i][17] = "*"
                        elif self.horus_matrix[i][16] < 0 and self.horus_matrix[i][15] > \
                                int(self.config.models_safe_interval):
                            self.horus_matrix[i][17] = initial



    def update_compound_predictions(self):
        '''
        updates the predictions based on the compound
        :return:
        '''
        self.sys.log.info(':: updating compounds predictions')
        i_sent = None
        i_first_word = None
        for i in range(len(self.horus_matrix)):
            if self.horus_matrix[i][7] == 1:
                y = self.horus_matrix[i][25]
                i_sent = self.horus_matrix[i][1]
                i_first_word = self.horus_matrix[i][2]
                c_size = int(self.horus_matrix[i][8])
            if self.horus_matrix[i][7] == 0 and i_sent is not None:
                if self.horus_matrix[i][1] == i_sent and self.horus_matrix[i][2] == i_first_word:
                    for k in range(c_size):
                        self.horus_matrix[i + k][25] = y
        self.sys.log.info(':: done')

    def process_input_text(self,text):
        self.sys.log.info(':: text: ' + text)
        self.sys.log.info(':: tokenizing sentences ...')
        sent_tokenize_list = sent_tokenize(text)
        self.sys.log.info(':: processing ' + str(len(sent_tokenize_list)) + ' sentence(s).')
        sentences = []
        w = []
        ner = []
        pos = []
        hasNER = -1
        for sentence in sent_tokenize_list:
            tokens = nltk.word_tokenize(sentence)
            pos_universal = nltk.pos_tag(tokens, tagset='universal')
            chunked = nltk.ne_chunk(nltk.pos_tag(tokens))
            for ch in chunked:
                if type(ch) is nltk.Tree:
                    w.append(ch[0][0])
                    pos.append(ch[0][1])
                    hasNER = 1
                    if ch._label == 'GPE' or ch._label == 'LOCATION':
                        ner.append('LOC')
                    elif ch._label == 'PERSON':
                        ner.append('PER')
                    elif ch._label == 'ORGANIZATION':
                        ner.append('ORG')
                    else:
                        ner.append('O')
                else:
                    w.append(ch[0])
                    pos.append(ch[1])
                    ner.append('O')

            _pos = list(zip(*pos_universal)[1])
            sentences.append([hasNER, sentence, tokens, ner, pos, _pos])
            pos = []
            ner = []
            w = []
        return sentences


    def process_ritter_ds(self,dspath):
        '''
        return a set of sentences
        :param dspath: path to Ritter dataset
        :return: sentence contains any entity?, sentence, words, NER tags
        '''
        sentences = []
        w = []
        t = []
        s = ''
        has3NER = -1
        with open(dspath) as f:
            for line in f:
                token = line.split('\t')[0]
                tag = line.split('\t')[1].replace('\r','').replace('\n','')
                if token == '':
                    if len(w) != 0:
                        pos_uni = nltk.pos_tag(w, tagset='universal')
                        pos = nltk.pos_tag(w)
                        _pos = zip(*pos)[1]
                        _pos_uni = zip(*pos_uni)[1]
                        sentences.append([has3NER, s, w, t, list(_pos), list(_pos_uni)])
                        w = []
                        t = []
                        s = ''
                        has3NER = -1
                else:
                    s += token + ' '
                    w.append(token)
                    t.append(tag)
                    if tag in definitions.NER_RITTER:
                        has3NER = 1
        return sentences